/*
    This file is part of BlueJoy.

    BlueJoy is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    BlueJoy is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with BlueJoy.  If not, see <http://www.gnu.org/licenses/>.
*/

package org.bluejoy;

import org.apache.log4j.BasicConfigurator;
import org.apache.log4j.FileAppender;
import org.apache.log4j.Logger;
import org.apache.log4j.PatternLayout;


public class CustomLogger
{
	//log4j
	private Logger logger = null;

	//singleton
	private static CustomLogger instance = null;
	
	//logger priority levels
	public static enum Level
	{
		DEBUG,
		INFO,
		WARN,
		ERROR,
		FATAL
	}
	
	//singleton constructor
	private CustomLogger()
	{
		//exists only to defeat instantiation
	}
	
	//singleton accessor
	public static CustomLogger getInstance()
	{
		if (instance == null)
		{
			instance = new CustomLogger();
		}
		
		return instance;
	}

	public void initLogger()
	{
		//root logger will act as an ancestor for other loggers
		logger = Logger.getRootLogger();
		
		//initialize log4j 
		String pattern =  "Milliseconds since program start: %r %n";
	    pattern += "Date in ISO8601 format: %d{ISO8601} %n";
	    pattern += "%m %n %n";
	    
	    FileAppender appender = null;
        try
        {
        	appender = new FileAppender(new PatternLayout(pattern),"log.txt");
        }
        catch(Exception e)
        {
        	System.out.println("Exception thrown while instantiating log4j appender.");
        	e.printStackTrace();
        }

        //configure log4j
        BasicConfigurator.configure();
        logger.addAppender(appender);
        logger.setLevel(org.apache.log4j.Level.WARN);	//we normally want this set to DEBUG
	}
	
	
	public void log(Level level, String message)
	{
		//get meta data for calling function
		int lineNumber = Thread.currentThread().getStackTrace()[2].getLineNumber();
		String methodName = Thread.currentThread().getStackTrace()[2].getMethodName();
		String className = Thread.currentThread().getStackTrace()[2].getClassName();
		String fileName = Thread.currentThread().getStackTrace()[2].getFileName();
		
		//assemble strings
		message = "Message: " + message;
		String locationInfo = "Location of log event: line " + lineNumber + ", in method \"" + methodName + "\", in file \"" + fileName + "\" \r\n";
		String classInfo = "Classname of caller: \"" + className + "\" \r\n";
		message = classInfo + locationInfo + message;

		//call log4j's log method
		if (level == Level.DEBUG)
			logger.log(org.apache.log4j.Level.DEBUG, message);
		else if (level == Level.INFO)
			logger.log(org.apache.log4j.Level.INFO, message);
		else if (level == Level.WARN)
			logger.log(org.apache.log4j.Level.WARN, message);
		else if (level == Level.ERROR)
			logger.log(org.apache.log4j.Level.ERROR, message);
		else if (level == Level.FATAL)
			logger.log(org.apache.log4j.Level.FATAL, message);
		else
		{
			logger.log(org.apache.log4j.Level.WARN, "Logging level provided is invalid.");
		}
	}
}
