/*
    This file is part of BlueJoy.

    BlueJoy is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    BlueJoy is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with BlueJoy.  If not, see <http://www.gnu.org/licenses/>.
*/

package org.bluejoy.controller;

import org.bluejoy.CustomLogger;
import org.bluejoy.model.Model;
import org.bluejoy.view.View;


public class Controller
{
	private CustomLogger logger;
	private Model model;
	private View view;
	private StatusBarController statusBarController;
	private RemoteDevicesController remoteDevicesController;
	private FileTransferTxController fileTransferTxController;
	private FileTransferRxController fileTransferRxController;
	
	//constructor
	public Controller()
	{
		//get a handle on logger
		logger = CustomLogger.getInstance();
		
		model = null;
		view = null;
        
		//add additional "specific controller" instances
		statusBarController = new StatusBarController();
		remoteDevicesController = new RemoteDevicesController();
		fileTransferTxController = new FileTransferTxController();
		fileTransferRxController = new FileTransferRxController();
	}
	
	public void init(Model model, View view)
	{
		this.model = model;
		this.view = view;
		
		//init specific controller instances
		statusBarController.init(model, view.getStatusBarPanel());
		remoteDevicesController.init(model, view.getRemoteDevicesPanel());
		fileTransferRxController.init(model, view.getFileTransferRxPanel());
		fileTransferTxController.init(model, view.getFileTransferTxPanel());
	}


	//getters
	public RemoteDevicesController getRemoteDevicesController() {
		return remoteDevicesController;
	}
	
	public FileTransferTxController getFileTransferTxController() {
		return fileTransferTxController;
	}

	public FileTransferRxController getFileTransferRxController() {
		return fileTransferRxController;
	}
}
