/*
    This file is part of BlueJoy.

    BlueJoy is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    BlueJoy is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with BlueJoy.  If not, see <http://www.gnu.org/licenses/>.
*/

package org.bluejoy.controller;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.JFileChooser;
import javax.swing.JOptionPane;

import org.bluejoy.model.BtDevice;
import org.bluejoy.model.CustomFileFilter;
import org.bluejoy.model.FileIO;
import org.bluejoy.model.Model;
import org.bluejoy.model.RemoteFile;
import org.bluejoy.model.Model.DeviceStatus;
import org.bluejoy.model.Model.FileTransferMethod;
import org.bluejoy.view.FileTransferRxPanel;



public class FileTransferRxController implements ActionListener
{
	private FileTransferRxPanel fileTransferRxPanel;
	private Model model;
	
	//constructor
	public FileTransferRxController()
	{
		fileTransferRxPanel = null;
		model = null;
	}
	
	//init
	public void init(Model model, FileTransferRxPanel fileTransferRxPanel)
	{
		this.model = model;
		this.fileTransferRxPanel = fileTransferRxPanel;
	}
	
	
	//ActionListener interface
	public void actionPerformed(ActionEvent e)
	{
		BtDevice selectedDevice = model.getSelectedDevice();
		
		//Browse button
		if (e.getSource().equals(fileTransferRxPanel.getBrowseButton()))
		{
			JFileChooser fileChooser = new JFileChooser();
			fileChooser.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
			fileChooser.addChoosableFileFilter(new CustomFileFilter());
			
			int returnValue = fileChooser.showOpenDialog(fileTransferRxPanel);
			if (returnValue == JFileChooser.APPROVE_OPTION)
			{
				File file = fileChooser.getSelectedFile();
				selectedDevice.setRxLocalFile(file);
			}
			
			model.dataChanged();
		}
		//Scan button
		else if (e.getSource().equals(fileTransferRxPanel.getScanButton()))
		{
			/*
			//XXX - debug that adds authentication request to all FTP connections
			String deviceURL = null;
			for (int i = 0; i < selectedDevice.getServices().size(); i++)
			{
				if ( selectedDevice.getServices().get(i).getServiceUUID().equals(Model.OBEX_FILE_TRANSFER) )
				{
					deviceURL = selectedDevice.getServices().get(i).getServiceUrl();
					deviceURL = deviceURL.substring(0, deviceURL.indexOf("authenticate"));
					deviceURL += "authenticate=true;encrypt=false;master=false";
					selectedDevice.getServices().get(i).setServiceUrl(deviceURL);
					break;
				}
			}
			*/
			
			//set busy status
			model.setDeviceStatus(DeviceStatus.FTP_TRANSFER);
			
			//clear tree data
			model.getSelectedDevice().clearRemoteFileSystemRoot();
			model.dataChanged();
			
			//scan
			model.browseRemoteDevice();
			
			//set idle status
			model.setDeviceStatus(DeviceStatus.IDLE);
			model.dataChanged();
		}
		//File Info button
		else if (e.getSource().equals(fileTransferRxPanel.getFileInfoButton()))
		{
			//check if something is actually selected first
			if (fileTransferRxPanel.getRemoteTree().isSelectionEmpty())
				return;
			
			DefaultMutableTreeNode node = (DefaultMutableTreeNode)fileTransferRxPanel.getRemoteTree().getSelectionPath().getLastPathComponent();
			String name = "name = " + ((RemoteFile)node.getUserObject()).getName();
			String dateCreated = "date created = " + ((RemoteFile)node.getUserObject()).getDateCreated();
			String memory = "memory type = " + ((RemoteFile)node.getUserObject()).getMemoryType();
			String path = "path = " + ((RemoteFile)node.getUserObject()).getPath();
			String permissions = "permissions = " + ((RemoteFile)node.getUserObject()).getPermissions();
			String size = "size = " + ((RemoteFile)node.getUserObject()).getSize();
			
			String message = "File Information: \n\n" + name + "\n" +
					dateCreated + "\n" + 
					memory + "\n" + 
					path + "\n" +
					permissions + "\n" +
					size;
					
			JOptionPane.showMessageDialog(fileTransferRxPanel,
					message,
					"File Info",
					JOptionPane.INFORMATION_MESSAGE,
					null);
		}
		//RX FTP radio button
		else if (e.getSource().equals(fileTransferRxPanel.getFtpRxRadioButton()))
		{
			selectedDevice.setFileTransferRxMethod(FileTransferMethod.FTP);
			model.dataChanged();
		}
		//RX SPP radio button
		else if (e.getSource().equals(fileTransferRxPanel.getSppRxRadioButton()))
		{
			selectedDevice.setFileTransferRxMethod(FileTransferMethod.SPP);
			model.dataChanged();
		}
		//Start Transfer button
		else if (e.getSource().equals(fileTransferRxPanel.getStartTransferButton()))
		{
			//validate the local directory path
			String localPath = fileTransferRxPanel.getLocalTextField().getText();
			if (FileIO.validateDirectory(localPath))
			{
				selectedDevice.setRxLocalFile(new File(localPath));
			}
			else
			{
				JOptionPane.showMessageDialog(fileTransferRxPanel,
						"Warning: Local directory path is invalid.",
						"Invalid Directory",
						JOptionPane.WARNING_MESSAGE,
						null);
				return;
			}
			
			//check that a remote file has actually been chosen
			if (fileTransferRxPanel.getRemoteTree().isSelectionEmpty())
			{
				JOptionPane.showMessageDialog(fileTransferRxPanel,
						"Warning: You must select a file on the remote BT device for transfer.",
						"Invalid Remote File Selection",
						JOptionPane.WARNING_MESSAGE,
						null);
				return;
			}
			
			//set busy status
			model.setDeviceStatus(DeviceStatus.FTP_TRANSFER);
			model.dataChanged();

			//receive data
			Object remotePathArray[] = fileTransferRxPanel.getRemoteTree().getSelectionPath().getPath();
			String remotePath = "/";
			if (remotePathArray.length > 2)
			{
				for (int i = 1; i < (remotePathArray.length - 1); i++)
					remotePath += remotePathArray[i] + "/";
			}
			remotePath = remotePath.substring(0, remotePath.length() - 1);	//remove '/' at the end of the string 
			selectedDevice.setRxTargetPath(remotePath);
			selectedDevice.setRxTargetName(remotePathArray[remotePathArray.length - 1].toString());
			model.receiveFileUsingFTP();
			
			//set status idle
			model.setDeviceStatus(DeviceStatus.IDLE);
			model.dataChanged();
		}
	}
}
