/*
    This file is part of BlueJoy.

    BlueJoy is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    BlueJoy is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with BlueJoy.  If not, see <http://www.gnu.org/licenses/>.
*/

package org.bluejoy.controller;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import javax.swing.JTable;

import org.bluejoy.model.Model;
import org.bluejoy.model.Model.DeviceStatus;
import org.bluejoy.view.RemoteDevicesPanel;



public class RemoteDevicesController implements ActionListener, MouseListener
{
	private RemoteDevicesPanel remoteDevicesPanel;
	private Model model;
	
	
	//constructor
	public RemoteDevicesController()
	{
		model = null;
		remoteDevicesPanel = null;
	}
	
	//init
	public void init(Model model, RemoteDevicesPanel remoteDevicesPanel)
	{
		this.model = model;
		this.remoteDevicesPanel = remoteDevicesPanel;
	}
	
	
	//ActionListener interface
	public void actionPerformed(ActionEvent e)
	{
		if (e.getSource().equals(remoteDevicesPanel.getScanButton()))
		{
			//clear current data/display
			model.clear();

			//set busy status
			model.setDeviceStatus(DeviceStatus.DEVICE_SEARCH);
			model.dataChanged();

			//scan
			model.scanForDevices();
			model.parseRemoteDevicesData();

			//set preferred file transfer methods
			model.setPreferredMethods();

			//set idle status
			model.setDeviceStatus(DeviceStatus.IDLE);
			model.dataChanged();
		}
	}
	

	//MouseListener interface
	public void mouseClicked(MouseEvent e)
	{
		if ( e.getSource().equals(remoteDevicesPanel.getDeviceTable()) )
		{
			//determine what was selected
			int selectedRow = ((JTable)(e.getSource())).getSelectedRow();
			if (selectedRow >= 0)
			{
				String deviceName = new String( (String)((JTable)(e.getSource())).getValueAt(selectedRow, 0) );
				model.setSelectedDevice(deviceName);
				model.setSelectedDeviceTableRow(selectedRow);
			}
			else
			{
				model.setSelectedDevice(null);
				model.setSelectedDeviceTableRow(-1);
			}
			
			model.dataChanged();
		}
	}

	public void mouseEntered(MouseEvent e) {
	}

	public void mouseExited(MouseEvent e) {
	}

	public void mousePressed(MouseEvent e) {
	}

	public void mouseReleased(MouseEvent e) {
	}
}
