/*
    This file is part of BlueJoy.

    BlueJoy is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    BlueJoy is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with BlueJoy.  If not, see <http://www.gnu.org/licenses/>.
*/

package org.bluejoy.model;

import java.util.ArrayList;
import javax.bluetooth.BluetoothStateException;
import javax.bluetooth.DeviceClass;
import javax.bluetooth.DiscoveryAgent;
import javax.bluetooth.DiscoveryListener;
import javax.bluetooth.LocalDevice;
import javax.bluetooth.RemoteDevice;
import javax.bluetooth.ServiceRecord;

import org.bluejoy.CustomLogger;
import org.bluejoy.CustomLogger.Level;
import org.bluejoy.model.Model.DeviceStatus;



public class DeviceDiscovery implements DiscoveryListener
{
	private CustomLogger logger;
	private Model model;
	
	//constructor
	public DeviceDiscovery()
	{
		//get a handle on logger
		logger  = CustomLogger.getInstance();
        model = null;
	}
	
	public void init(Model model)
	{
		this.model = model;
	}

	
	public void deviceDiscovered(RemoteDevice btDevice, DeviceClass cod)
	{
		logger.log(Level.DEBUG, "Device " + btDevice.getBluetoothAddress() + " found");
		addRemoteDevice(btDevice, cod);
    }

    public void inquiryCompleted(int discType)
    {
    	logger.log(Level.DEBUG, "Device Inquiry completed!");
    	model.setDeviceStatus(DeviceStatus.IDLE);
        for (int i = 0; i < model.getRemoteDevices().size(); i++)
        	model.getRemoteDevices().get(i).parseRemoteDevice();
	}

    public void serviceSearchCompleted(int transID, int respCode) {
    }

    public void servicesDiscovered(int transID, ServiceRecord[] serviceRecord) {
    }
    

    //DiscoveryAgent methods
    public void startDeviceSearch()
    {
    	try
    	{
			boolean result = LocalDevice.getLocalDevice().getDiscoveryAgent().startInquiry(DiscoveryAgent.GIAC, this);
			if (result == true)
				model.setDeviceStatus(DeviceStatus.DEVICE_SEARCH);
		}
    	catch (BluetoothStateException bse)
    	{
    		logger.log(Level.ERROR, "Exception thrown while starting a BT device search.");
			bse.printStackTrace();
			model.setDeviceStatus(DeviceStatus.IDLE);
		}
    }
    
    
	//utility methods
	public void addRemoteDevice(RemoteDevice remoteDevice, DeviceClass cod)
	{
		ArrayList<BtDevice> remoteDevices = model.getRemoteDevices();
		
		//check if remoteDevice is in the ArrayList already
		for (int i = 0; i < remoteDevices.size(); i++)
		{
			if (remoteDevices.get(i).getRemoteDevice().getBluetoothAddress().equals(remoteDevice.getBluetoothAddress()))
			{
				//found a matching BT address
				return;
			}
		}
		
		//no matches exist in ArrayList, so add the new device
		BtDevice btDevice = new BtDevice(remoteDevice, cod);
		remoteDevices.add(btDevice);
	}
}
