/*
    This file is part of BlueJoy.

    BlueJoy is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    BlueJoy is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with BlueJoy.  If not, see <http://www.gnu.org/licenses/>.
*/

package org.bluejoy.model;

import java.io.File;
import java.io.IOException;
import java.io.OutputStream;
import javax.microedition.io.Connector;
import javax.obex.ClientSession;
import javax.obex.HeaderSet;
import javax.obex.Operation;
import javax.obex.ResponseCodes;

import org.bluejoy.CustomLogger;
import org.bluejoy.CustomLogger.Level;


public class OppClient
{
	private CustomLogger logger;
	private Model model;
	
	//constructor
	public OppClient()
	{
		//get a handle on logger
		logger = CustomLogger.getInstance();
		model = null;
	}
	
	public void init(Model model)
	{
		this.model = model;
	}
	
	//OPP file transfer methods
	public void oppPush(File file)
	{
		//verify that a BT device is selected
		if (model.getSelectedDevice() == null)
		{
			logger.log(Level.ERROR, "No Bluetooth device selected for OPP put operation");
			return;
		}
		
		//get the device URL of the Object Push Profile for the selected device
		String deviceURL = null;
		for (int i = 0; i < model.getSelectedDevice().getServices().size(); i++)
		{
			if ( model.getSelectedDevice().getServices().get(i).getServiceUUID().equals(Model.OBEX_OBJECT_PUSH) )
			{
				deviceURL = model.getSelectedDevice().getServices().get(i).getServiceUrl();
				break;
			}
		}
		if (deviceURL == null)
		{
			logger.log(Level.ERROR, "OPP profile not found for selected device");
			return;
		}

		//create a Client Session (i.e. socket) to the deviceURL
		ClientSession clientSession = null;
		try 
		{
			clientSession = (ClientSession) Connector.open(deviceURL);
		}
		catch (IOException ioe)
		{
			logger.log(Level.ERROR, "Exception thrown while opening device URL = " + deviceURL);
			return;
		}
		if (clientSession == null)
		{
			logger.log(Level.ERROR, "Failed to open device URL = " + deviceURL);
			return;
		}
		
		//connect to the Client Session (i.e. socket)
		HeaderSet hsConnectReply = null;
      	try
      	{
	      	hsConnectReply = clientSession.connect(null);
			if (hsConnectReply.getResponseCode() != ResponseCodes.OBEX_HTTP_OK) {
				logger.log(Level.ERROR, "Failed to connect to client session");
				clientSession.close();
				return;
			}
      	}
      	catch (IOException ioe)
      	{
      		logger.log(Level.ERROR, "Exception thrown while attempting to connect to client session");
			ioe.printStackTrace();
			return;
		}

        //load data to be sent
        byte data[] = FileIO.getBytesFromFile(file);

		//create header       
        HeaderSet hsOperation = clientSession.createHeaderSet();
        hsOperation.setHeader(HeaderSet.NAME, file.getName());
        hsOperation.setHeader(HeaderSet.TYPE, "binary");
        hsOperation.setHeader(HeaderSet.LENGTH, (long)data.length);

        //create PUT Operation
        Operation putOperation = null;
        OutputStream os = null;
		try
		{
			//send the data
			putOperation = clientSession.put(hsOperation);
        	os = putOperation.openOutputStream();
        	os.write(data);
        	os.close();
        	putOperation.close();
        }
		catch (Exception e)
		{
			logger.log(Level.ERROR, "Exception thrown while sending binary data to remote BT device");
        	e.printStackTrace();
        }
		
		//tear down client session
		try
		{
			clientSession.disconnect(null);
            clientSession.close();
		}
		catch (IOException ioe)
		{
			logger.log(Level.ERROR, "Exception thrown while tearing down client session");
			ioe.printStackTrace();
		}
	}
}
