/*
    This file is part of BlueJoy.

    BlueJoy is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    BlueJoy is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with BlueJoy.  If not, see <http://www.gnu.org/licenses/>.
*/

package org.bluejoy.model;

import java.util.ArrayList;
import javax.bluetooth.BluetoothStateException;
import javax.bluetooth.DeviceClass;
import javax.bluetooth.DiscoveryListener;
import javax.bluetooth.LocalDevice;
import javax.bluetooth.RemoteDevice;
import javax.bluetooth.ServiceRecord;
import javax.bluetooth.UUID;

import org.bluejoy.CustomLogger;
import org.bluejoy.CustomLogger.Level;
import org.bluejoy.model.Model.DeviceStatus;



public class ServiceDiscovery implements DiscoveryListener
{
	private CustomLogger logger;
	private Model model;
	
	//constructor
	public ServiceDiscovery()
	{
		//get a handle on logger
		logger = CustomLogger.getInstance();
		
		model = null;
	}
	
	public void init(Model model)
	{
		this.model = model;
	}

	
	//DiscoveryListener interface
	public void deviceDiscovered(RemoteDevice btDevice, DeviceClass cod) {
	}

	public void inquiryCompleted(int discType) {
	}

	public void serviceSearchCompleted(int transID, int respCode)
	{
		model.setDeviceStatus(DeviceStatus.IDLE);
	}

	public void servicesDiscovered(int transID, ServiceRecord[] serviceRecord)
	{
		addServiceRecord(serviceRecord[0].getHostDevice(), serviceRecord);
	}
	
	
	//DiscoveryAgent methods
	public void startServiceSearch(RemoteDevice btDevice)
	{
		UUID[] searchUuidSet = new UUID[] { Model.RFCOMM };
		int[] attrIDs =  new int[] {
	            0x0100	// service name
	    };
	    
		try
		{
			LocalDevice.getLocalDevice().getDiscoveryAgent().searchServices(attrIDs, searchUuidSet, btDevice, this);
			model.setDeviceStatus(DeviceStatus.SERVICE_SEARCH);
		}
		catch (BluetoothStateException bse)
		{
			logger.log(Level.ERROR, "Exception thrown while starting BT service search.");
			bse.printStackTrace();
			model.setDeviceStatus(DeviceStatus.IDLE);
		}
	}
	
	
	//utility methods
	public void addServiceRecord(RemoteDevice remoteDevice, ServiceRecord[] serviceRecord)
	{
		ArrayList<BtDevice> remoteDevices = model.getRemoteDevices();
		
		//look for a device with matching MAC address and update its service record
		for (int i = 0; i < remoteDevices.size(); i++)
		{
			if (remoteDevices.get(i).getRemoteDevice().getBluetoothAddress().equals(remoteDevice.getBluetoothAddress()))
			{
				remoteDevices.get(i).setServiceRecord(serviceRecord);
				return;
			}
		}
	}
}
