/*
    This file is part of BlueJoy.

    BlueJoy is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    BlueJoy is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with BlueJoy.  If not, see <http://www.gnu.org/licenses/>.
*/

package org.bluejoy.view;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import javax.swing.ButtonGroup;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JScrollPane;
import javax.swing.ScrollPaneConstants;
import javax.swing.JTextField;
import javax.swing.JTree;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.TreeSelectionModel;

import org.bluejoy.controller.Controller;
import org.bluejoy.model.BtDevice;
import org.bluejoy.model.Model;
import org.bluejoy.model.RemoteTreeNode;
import org.bluejoy.model.Model.FileTransferMethod;



public class FileTransferRxPanel extends JPanel
{
	private static final long serialVersionUID = 9058920331868893795L;
	private Model model;
	
	private JLabel localDeviceLabel;
	private JLabel remoteDeviceLabel;
	private JTextField localTextField;
	private JTree remoteTree;
	private JButton browseButton;
	private JButton scanButton;
	private JButton fileInfoButton;
	private JLabel rxMethodLabel;
	private JButton startTransferButton;
	private ButtonGroup rxButtonGroup;
	private JRadioButton ftpRxRadioButton;
	private JRadioButton sppRxRadioButton;
	
	
	
	public FileTransferRxPanel()
	{
		super(true);	//enable double-buffering
		
		model = null;
		
		//setup GUI
		GridBagLayout layout = new GridBagLayout();
        GridBagConstraints constraints = new GridBagConstraints();
        setLayout(layout);
        
        remoteDeviceLabel = new JLabel("Remote Filename");
    	constraints.gridx = 0;
        constraints.gridy = 0;
        constraints.gridwidth = 1;
        constraints.gridheight = 1;
        constraints.fill = GridBagConstraints.NONE;
        constraints.ipadx = 0;
        constraints.ipady = 0;
        constraints.insets = new Insets(5, 5, 5, 5);		//order is = top, left, bottom, right
        constraints.weightx = 0;
        constraints.weighty = 0;
        constraints.anchor = GridBagConstraints.NORTHWEST;
        layout.setConstraints(remoteDeviceLabel, constraints);
        add(remoteDeviceLabel);
        
        remoteTree = new JTree();
        remoteTree.setModel(null);
        remoteTree.getSelectionModel().setSelectionMode(TreeSelectionModel.SINGLE_TREE_SELECTION);
        JScrollPane treeScrollPane = new JScrollPane(remoteTree, ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED, ScrollPaneConstants.HORIZONTAL_SCROLLBAR_AS_NEEDED);
    	constraints.gridx = 0;
        constraints.gridy = 1;
        constraints.gridwidth = 1;
        constraints.gridheight = 3;
        constraints.fill = GridBagConstraints.BOTH;
        constraints.ipadx = 0;
        constraints.ipady = 0;
        constraints.insets = new Insets(5, 5, 5, 5);		//order is = top, left, bottom, right
        constraints.weightx = 1;	//at least one component's weight values have to be set per row; otherwise, all the components clump together in the middle
        constraints.weighty = 1;	//at least one component's Y weight value has to be set; otherwise, all the components clump together in the middle
        constraints.anchor = GridBagConstraints.NORTHWEST;
        layout.setConstraints(treeScrollPane, constraints);
        add(treeScrollPane);
        
        scanButton = new JButton("Scan");
    	constraints.gridx = 1;
        constraints.gridy = 1;
        constraints.gridwidth = 1;
        constraints.gridheight = 1;
        constraints.fill = GridBagConstraints.NONE;
        constraints.ipadx = 0;
        constraints.ipady = 0;
        constraints.insets = new Insets(5, 5, 5, 5);		//order is = top, left, bottom, right
        constraints.weightx = 0;
        constraints.weighty = 0;
        constraints.anchor = GridBagConstraints.NORTHWEST;
        layout.setConstraints(scanButton, constraints);
        add(scanButton);
        
        fileInfoButton = new JButton("File Info");
    	constraints.gridx = 1;
        constraints.gridy = 2;
        constraints.gridwidth = 1;
        constraints.gridheight = 1;
        constraints.fill = GridBagConstraints.NONE;
        constraints.ipadx = 0;
        constraints.ipady = 0;
        constraints.insets = new Insets(5, 5, 5, 5);		//order is = top, left, bottom, right
        constraints.weightx = 0;
        constraints.weighty = 0;
        constraints.anchor = GridBagConstraints.NORTHWEST;
        layout.setConstraints(fileInfoButton, constraints);
        add(fileInfoButton);
        
    	localDeviceLabel = new JLabel("Local Directory");
    	constraints.gridx = 0;
        constraints.gridy = 4;
        constraints.gridwidth = 2;
        constraints.gridheight = 1;
        constraints.fill = GridBagConstraints.NONE;
        constraints.ipadx = 0;
        constraints.ipady = 0;
        constraints.insets = new Insets(5, 5, 5, 5);		//order is = top, left, bottom, right
        constraints.weightx = 0;
        constraints.weighty = 0;
        constraints.anchor = GridBagConstraints.NORTHWEST;
        layout.setConstraints(localDeviceLabel, constraints);
        add(localDeviceLabel);
        
    	localTextField = new JTextField();
    	constraints.gridx = 0;
        constraints.gridy = 5;
        constraints.gridwidth = 1;
        constraints.gridheight = 1;
        constraints.fill = GridBagConstraints.HORIZONTAL;
        constraints.ipadx = 0;
        constraints.ipady = 0;
        constraints.insets = new Insets(5, 5, 5, 5);		//order is = top, left, bottom, right
        constraints.weightx = 0;
        constraints.weighty = 0;
        constraints.anchor = GridBagConstraints.NORTHWEST;
        layout.setConstraints(localTextField, constraints);
        add(localTextField);
        
        browseButton = new JButton("Browse");
    	constraints.gridx = 1;
        constraints.gridy = 5;
        constraints.gridwidth = 1;
        constraints.gridheight = 1;
        constraints.fill = GridBagConstraints.NONE;
        constraints.ipadx = 0;
        constraints.ipady = 0;
        constraints.insets = new Insets(5, 5, 5, 5);		//order is = top, left, bottom, right
        constraints.weightx = 0;
        constraints.weighty = 0;
        constraints.anchor = GridBagConstraints.NORTHWEST;
        layout.setConstraints(browseButton, constraints);
        add(browseButton);
        
        
        rxMethodLabel = new JLabel("RX Method");
    	constraints.gridx = 0;
        constraints.gridy = 6;
        constraints.gridwidth = 1;
        constraints.gridheight = 1;
        constraints.fill = GridBagConstraints.NONE;
        constraints.ipadx = 0;
        constraints.ipady = 0;
        constraints.insets = new Insets(5, 5, 0, 5);		//order is = top, left, bottom, right
        constraints.weightx = 0;
        constraints.weighty = 0;
        constraints.anchor = GridBagConstraints.NORTHWEST;
        layout.setConstraints(rxMethodLabel, constraints);
        add(rxMethodLabel);
    	
    	startTransferButton = new JButton("Start Transfer");
    	constraints.gridx = 1;
        constraints.gridy = 6;
        constraints.gridwidth = 1;
        constraints.gridheight = 1;
        constraints.fill = GridBagConstraints.NONE;
        constraints.ipadx = 0;
        constraints.ipady = 0;
        constraints.insets = new Insets(5, 5, 0, 5);		//order is = top, left, bottom, right
        constraints.weightx = 0;
        constraints.weighty = 0;
        constraints.anchor = GridBagConstraints.NORTHWEST;
        layout.setConstraints(startTransferButton, constraints);
        add(startTransferButton);
		
		ftpRxRadioButton = new JRadioButton("FTP");
    	constraints.gridx = 0;
        constraints.gridy = 7;
        constraints.gridwidth = 1;
        constraints.gridheight = 1;
        constraints.fill = GridBagConstraints.NONE;
        constraints.ipadx = 0;
        constraints.ipady = 0;
        constraints.insets = new Insets(5, 5, 0, 5);		//order is = top, left, bottom, right
        constraints.weightx = 0;
        constraints.weighty = 0;
        constraints.anchor = GridBagConstraints.NORTHWEST;
        layout.setConstraints(ftpRxRadioButton, constraints);
        add(ftpRxRadioButton);
    	
    	sppRxRadioButton = new JRadioButton("SPP");
    	constraints.gridx = 0;
        constraints.gridy = 8;
        constraints.gridwidth = 1;
        constraints.gridheight = 1;
        constraints.fill = GridBagConstraints.NONE;
        constraints.ipadx = 0;
        constraints.ipady = 0;
        constraints.insets = new Insets(5, 5, 0, 5);		//order is = top, left, bottom, right
        constraints.weightx = 0;
        constraints.weighty = 0;
        constraints.anchor = GridBagConstraints.NORTHWEST;
        layout.setConstraints(sppRxRadioButton, constraints);
        add(sppRxRadioButton);
        
        rxButtonGroup = new ButtonGroup();
		rxButtonGroup.add(ftpRxRadioButton);
		rxButtonGroup.add(sppRxRadioButton);
     }

	public void init(Model model, Controller controller)
	{
		this.model = model;
		
		//add event listeners
		scanButton.addActionListener(controller.getFileTransferRxController());
		fileInfoButton.addActionListener(controller.getFileTransferRxController());
		browseButton.addActionListener(controller.getFileTransferRxController());
		ftpRxRadioButton.addActionListener(controller.getFileTransferRxController());
		sppRxRadioButton.addActionListener(controller.getFileTransferRxController());
		startTransferButton.addActionListener(controller.getFileTransferRxController());
		
		ftpRxRadioButton.setSelected(true);
		ftpRxRadioButton.setEnabled(false);
		sppRxRadioButton.setSelected(false);
    	sppRxRadioButton.setEnabled(false);
	}

	
	public void update()
	{
		//refresh the info to be displayed
		
		//first, verify that a device is actually selected
		BtDevice device = model.getSelectedDevice();
		if (device != null)
		{
			//add JTree data
			RemoteTreeNode root = device.getRemoteFileSystemRoot();
			if (root == null)
				remoteTree.setModel(null);
			else
			{
				//in case either:
				//1) root != null and tree == null, then set tree data
				//2) root != null and tree root != root, then refresh/reset tree data
				if (remoteTree.getModel() == null || ((RemoteTreeNode)remoteTree.getModel().getRoot()).equals(root) == false)
					remoteTree.setModel(new DefaultTreeModel(root));
			}
			
			//add local directory string
			if (device.getRxLocalFile() != null)
				localTextField.setText(device.getRxLocalFile().getAbsolutePath());
			else
				localTextField.setText("");
			
			//select RX method
			if (device.getFileTransferRxMethod().equals(FileTransferMethod.FTP))
				rxButtonGroup.setSelected(ftpRxRadioButton.getModel(), true);
			else if (device.getFileTransferRxMethod().equals(FileTransferMethod.SPP))
				rxButtonGroup.setSelected(sppRxRadioButton.getModel(), true);
			
			//enable/disable RX methods
			if (device.isFileTransferSupported(FileTransferMethod.FTP))
				ftpRxRadioButton.setEnabled(true);
			else
				ftpRxRadioButton.setEnabled(false);

			//XXX - SPP is not yet implemented, so it is always disabled
			sppRxRadioButton.setEnabled(false);
			/*
			if (device.isFileTransferSupported(FileTransferMethod.SPP))
				sppRxRadioButton.setEnabled(true);
			else
				sppRxRadioButton.setEnabled(false);
			*/
			
			//enable/disable controls
			if (device.getFileTransferRxMethod().equals(FileTransferMethod.FTP))
			{
				//enable tree, scan, and file info
				remoteTree.setEnabled(true);
				scanButton.setEnabled(true);
				fileInfoButton.setEnabled(true);
			}
			else
			{
				//disable tree, scan, and file info
				remoteTree.setEnabled(false);
				scanButton.setEnabled(false);
				fileInfoButton.setEnabled(false);
			}
		}
	}

	
	//getters
	public JTextField getLocalTextField() {
		return localTextField;
	}

	public JTree getRemoteTree() {
		return remoteTree;
	}

	public JButton getBrowseButton() {
		return browseButton;
	}

	public JButton getScanButton() {
		return scanButton;
	}

	public JButton getFileInfoButton() {
		return fileInfoButton;
	}

	public JButton getStartTransferButton() {
		return startTransferButton;
	}

	public ButtonGroup getRxButtonGroup() {
		return rxButtonGroup;
	}

	public JRadioButton getFtpRxRadioButton() {
		return ftpRxRadioButton;
	}

	public JRadioButton getSppRxRadioButton() {
		return sppRxRadioButton;
	}
}
