/*
    This file is part of BlueJoy.

    BlueJoy is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    BlueJoy is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with BlueJoy.  If not, see <http://www.gnu.org/licenses/>.
*/

package org.bluejoy.view;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import javax.swing.ButtonGroup;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JScrollPane;
import javax.swing.JTextField;
import javax.swing.JTree;
import javax.swing.ScrollPaneConstants;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.TreeSelectionModel;

import org.bluejoy.controller.Controller;
import org.bluejoy.model.BtDevice;
import org.bluejoy.model.Model;
import org.bluejoy.model.RemoteTreeNode;
import org.bluejoy.model.Model.FileTransferMethod;



public class FileTransferTxPanel extends JPanel
{
	private static final long serialVersionUID = 276766901486779200L;
	private Model model;
	
	private JLabel supportedTypesLabel;
	private JLabel localDeviceLabel;
	private JLabel remoteDeviceLabel;
	private JTextField localTextField;
	private JTree remoteTree;
	private JButton browseButton;
	private JButton scanButton;
	private JLabel txMethodLabel;
	private JButton startTransferButton;
	private ButtonGroup txButtonGroup;
	private JRadioButton ftpTxRadioButton;
	private JRadioButton oppTxRadioButton;
	private JRadioButton sppTxRadioButton;

	
	public FileTransferTxPanel()
	{
		super(true);	//enable double-buffering
		model = null;
		
		//setup GUI
		GridBagLayout layout = new GridBagLayout();
        GridBagConstraints constraints = new GridBagConstraints();
        setLayout(layout);

        supportedTypesLabel = new JLabel("Supported file types include: jpg, png, gif, mp3");
        constraints.gridx = 0;
        constraints.gridy = 0;
        constraints.gridwidth = 2;
        constraints.gridheight = 1;
        constraints.fill = GridBagConstraints.NONE;
        constraints.ipadx = 0;
        constraints.ipady = 0;
        constraints.insets = new Insets(5, 5, 5, 5);		//order is = top, left, bottom, right
        constraints.weightx = 0;
        constraints.weighty = 0;
        constraints.anchor = GridBagConstraints.NORTHWEST;
        layout.setConstraints(supportedTypesLabel, constraints);
        add(supportedTypesLabel);
        
    	localDeviceLabel = new JLabel("Local Filename");
    	constraints.gridx = 0;
        constraints.gridy = 1;
        constraints.gridwidth = 2;
        constraints.gridheight = 1;
        constraints.fill = GridBagConstraints.NONE;
        constraints.ipadx = 0;
        constraints.ipady = 0;
        constraints.insets = new Insets(5, 5, 5, 5);		//order is = top, left, bottom, right
        constraints.weightx = 0;
        constraints.weighty = 0;
        constraints.anchor = GridBagConstraints.NORTHWEST;
        layout.setConstraints(localDeviceLabel, constraints);
        add(localDeviceLabel);
        
    	localTextField = new JTextField();
    	constraints.gridx = 0;
        constraints.gridy = 2;
        constraints.gridwidth = 1;
        constraints.gridheight = 1;
        constraints.fill = GridBagConstraints.HORIZONTAL;
        constraints.ipadx = 0;
        constraints.ipady = 0;
        constraints.insets = new Insets(5, 5, 5, 5);		//order is = top, left, bottom, right
        constraints.weightx = 0;
        constraints.weighty = 0;
        constraints.anchor = GridBagConstraints.NORTHWEST;
        layout.setConstraints(localTextField, constraints);
        add(localTextField);
        
        browseButton = new JButton("Browse");
    	constraints.gridx = 1;
        constraints.gridy = 2;
        constraints.gridwidth = 1;
        constraints.gridheight = 1;
        constraints.fill = GridBagConstraints.NONE;
        constraints.ipadx = 0;
        constraints.ipady = 0;
        constraints.insets = new Insets(5, 5, 5, 5);		//order is = top, left, bottom, right
        constraints.weightx = 0;
        constraints.weighty = 0;
        constraints.anchor = GridBagConstraints.NORTHWEST;
        layout.setConstraints(browseButton, constraints);
        add(browseButton);
        
        remoteDeviceLabel = new JLabel("Destination Directory");
    	constraints.gridx = 0;
        constraints.gridy = 3;
        constraints.gridwidth = 1;
        constraints.gridheight = 1;
        constraints.fill = GridBagConstraints.NONE;
        constraints.ipadx = 0;
        constraints.ipady = 0;
        constraints.insets = new Insets(5, 5, 5, 5);		//order is = top, left, bottom, right
        constraints.weightx = 0;
        constraints.weighty = 0;
        constraints.anchor = GridBagConstraints.NORTHWEST;
        layout.setConstraints(remoteDeviceLabel, constraints);
        add(remoteDeviceLabel);
        
        //remoteTree = new JTree(new DefaultMutableTreeNode("unknown"));
        remoteTree = new JTree();
        remoteTree.setModel(null);
        remoteTree.getSelectionModel().setSelectionMode(TreeSelectionModel.SINGLE_TREE_SELECTION);
        JScrollPane treeScrollPane = new JScrollPane(remoteTree, ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED, ScrollPaneConstants.HORIZONTAL_SCROLLBAR_AS_NEEDED);
    	constraints.gridx = 0;
        constraints.gridy = 4;
        constraints.gridwidth = 1;
        constraints.gridheight = 2;
        constraints.fill = GridBagConstraints.BOTH;
        constraints.ipadx = 0;
        constraints.ipady = 0;
        constraints.insets = new Insets(5, 5, 5, 5);		//order is = top, left, bottom, right
        constraints.weightx = 1;	//at least one component's weight values have to be set per row; otherwise, all the components clump together in the middle
        constraints.weighty = 1;	//at least one component's Y weight value has to be set; otherwise, all the components clump together in the middle
        constraints.anchor = GridBagConstraints.NORTHWEST;
        layout.setConstraints(treeScrollPane, constraints);
        add(treeScrollPane);
        
        scanButton = new JButton("Scan");
    	constraints.gridx = 1;
        constraints.gridy = 4;
        constraints.gridwidth = 1;
        constraints.gridheight = 1;
        constraints.fill = GridBagConstraints.NONE;
        constraints.ipadx = 0;
        constraints.ipady = 0;
        constraints.insets = new Insets(5, 5, 5, 5);		//order is = top, left, bottom, right
        constraints.weightx = 0;
        constraints.weighty = 0;
        constraints.anchor = GridBagConstraints.NORTHWEST;
        layout.setConstraints(scanButton, constraints);
        add(scanButton);
        
    	txMethodLabel = new JLabel("TX Method");
    	constraints.gridx = 0;
        constraints.gridy = 6;
        constraints.gridwidth = 1;
        constraints.gridheight = 1;
        constraints.fill = GridBagConstraints.NONE;
        constraints.ipadx = 0;
        constraints.ipady = 0;
        constraints.insets = new Insets(5, 5, 0, 5);		//order is = top, left, bottom, right
        constraints.weightx = 0;
        constraints.weighty = 0;
        constraints.anchor = GridBagConstraints.NORTHWEST;
        layout.setConstraints(txMethodLabel, constraints);
        add(txMethodLabel);
    	
    	startTransferButton = new JButton("Start Transfer");
    	constraints.gridx = 1;
        constraints.gridy = 6;
        constraints.gridwidth = 1;
        constraints.gridheight = 1;
        constraints.fill = GridBagConstraints.NONE;
        constraints.ipadx = 0;
        constraints.ipady = 0;
        constraints.insets = new Insets(5, 5, 0, 5);		//order is = top, left, bottom, right
        constraints.weightx = 0;
        constraints.weighty = 0;
        constraints.anchor = GridBagConstraints.NORTHWEST;
        layout.setConstraints(startTransferButton, constraints);
        add(startTransferButton);
        
    	ftpTxRadioButton = new JRadioButton("FTP");
    	constraints.gridx = 0;
        constraints.gridy = 7;
        constraints.gridwidth = 1;
        constraints.gridheight = 1;
        constraints.fill = GridBagConstraints.NONE;
        constraints.ipadx = 0;
        constraints.ipady = 0;
        constraints.insets = new Insets(5, 5, 0, 5);		//order is = top, left, bottom, right
        constraints.weightx = 0;
        constraints.weighty = 0;
        constraints.anchor = GridBagConstraints.NORTHWEST;
        layout.setConstraints(ftpTxRadioButton, constraints);
        add(ftpTxRadioButton);
        
    	oppTxRadioButton = new JRadioButton("OPP");
    	constraints.gridx = 0;
        constraints.gridy = 8;
        constraints.gridwidth = 1;
        constraints.gridheight = 1;
        constraints.fill = GridBagConstraints.NONE;
        constraints.ipadx = 0;
        constraints.ipady = 0;
        constraints.insets = new Insets(5, 5, 0, 5);		//order is = top, left, bottom, right
        constraints.weightx = 0;
        constraints.weighty = 0;
        constraints.anchor = GridBagConstraints.NORTHWEST;
        layout.setConstraints(oppTxRadioButton, constraints);
        add(oppTxRadioButton);
    	
    	sppTxRadioButton = new JRadioButton("SPP");
    	constraints.gridx = 0;
        constraints.gridy = 9;
        constraints.gridwidth = 1;
        constraints.gridheight = 1;
        constraints.fill = GridBagConstraints.NONE;
        constraints.ipadx = 0;
        constraints.ipady = 0;
        constraints.insets = new Insets(5, 5, 0, 5);		//order is = top, left, bottom, right
        constraints.weightx = 0;
        constraints.weighty = 0;	
        constraints.anchor = GridBagConstraints.NORTHWEST;
        layout.setConstraints(sppTxRadioButton, constraints);
        add(sppTxRadioButton);
        
        txButtonGroup = new ButtonGroup();
		txButtonGroup.add(ftpTxRadioButton);
		txButtonGroup.add(oppTxRadioButton);
		txButtonGroup.add(sppTxRadioButton);
     }

	public void init(Model model, Controller controller)
	{
		this.model = model;
		
		//add event listeners
		browseButton.addActionListener(controller.getFileTransferTxController());
		scanButton.addActionListener(controller.getFileTransferTxController());
		ftpTxRadioButton.addActionListener(controller.getFileTransferTxController());
		oppTxRadioButton.addActionListener(controller.getFileTransferTxController());
		sppTxRadioButton.addActionListener(controller.getFileTransferTxController());
		startTransferButton.addActionListener(controller.getFileTransferTxController());
		
		remoteTree.setEnabled(false);
		scanButton.setEnabled(false);
		ftpTxRadioButton.setSelected(false);
    	ftpTxRadioButton.setEnabled(false);
    	oppTxRadioButton.setSelected(true);
    	oppTxRadioButton.setEnabled(false);
    	sppTxRadioButton.setSelected(false);
    	sppTxRadioButton.setEnabled(false);
	}

	
	public void update()
	{
		//refresh the info to be displayed
		
		//first, verify that a device is actually selected
		BtDevice device = model.getSelectedDevice();
		if (device != null)
		{
			//add local directory string
			if (device.getTxLocalFile() != null)
				localTextField.setText(device.getTxLocalFile().getAbsolutePath());
			else
				localTextField.setText("");
			
			//add JTree data
			RemoteTreeNode root = device.getRemoteFileSystemRoot();
			if (root == null)
				remoteTree.setModel(null);
			else
			{
				if (remoteTree.getModel() == null || ((RemoteTreeNode)remoteTree.getModel().getRoot()).equals(root) == false)
					remoteTree.setModel(new DefaultTreeModel(root));
			}
			
			//select TX method
			if (device.getFileTransferTxMethod().equals(FileTransferMethod.OPP))
				txButtonGroup.setSelected(oppTxRadioButton.getModel(), true);
			else if (device.getFileTransferTxMethod().equals(FileTransferMethod.FTP))
				txButtonGroup.setSelected(ftpTxRadioButton.getModel(), true);
			else if (device.getFileTransferTxMethod().equals(FileTransferMethod.SPP))
				txButtonGroup.setSelected(sppTxRadioButton.getModel(), true);
			
			//enable/disable TX methods
			if (device.isFileTransferSupported(FileTransferMethod.OPP))
				oppTxRadioButton.setEnabled(true);
			else
				oppTxRadioButton.setEnabled(false);
			
			if (device.isFileTransferSupported(FileTransferMethod.FTP))
				ftpTxRadioButton.setEnabled(true);
			else
				ftpTxRadioButton.setEnabled(false);
			
			//XXX - SPP is not yet implemented, so it is always disabled
			sppTxRadioButton.setEnabled(false);
			/*
			if (device.isFileTransferSupported(FileTransferMethod.SPP))
				this.sppTxRadioButton.setEnabled(true);
			else
				this.sppTxRadioButton.setEnabled(false);
			*/
			
			//enable/disable controls
			if (device.getFileTransferTxMethod().equals(FileTransferMethod.FTP))
			{
				//enable tree and scan
				remoteTree.setEnabled(true);
				scanButton.setEnabled(true);
			}
			else
			{
				//disable tree and scan
				remoteTree.setEnabled(false);
				scanButton.setEnabled(false);
			}
		}
	}

	
	//getters
	public static long getSerialVersionUID() {
		return serialVersionUID;
	}

	public JTextField getLocalTextField() {
		return localTextField;
	}

	public JTree getRemoteTree() {
		return remoteTree;
	}

	public JButton getBrowseButton() {
		return browseButton;
	}

	public JButton getScanButton() {
		return scanButton;
	}

	public JButton getStartTransferButton() {
		return startTransferButton;
	}

	public ButtonGroup getTxButtonGroup() {
		return txButtonGroup;
	}

	public JRadioButton getFtpTxRadioButton() {
		return ftpTxRadioButton;
	}

	public JRadioButton getOppTxRadioButton() {
		return oppTxRadioButton;
	}

	public JRadioButton getSppTxRadioButton() {
		return sppTxRadioButton;
	}
}
