/*
    This file is part of BlueJoy.

    BlueJoy is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    BlueJoy is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with BlueJoy.  If not, see <http://www.gnu.org/licenses/>.
*/

package org.bluejoy.view;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.util.Vector;
import javax.swing.JButton;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.JToolBar;
import javax.swing.ListSelectionModel;
import javax.swing.table.DefaultTableModel;

import org.bluejoy.controller.Controller;
import org.bluejoy.model.Model;
import org.bluejoy.model.Model.DeviceStatus;



public class RemoteDevicesPanel extends JPanel 
{
	private static final long serialVersionUID = -656083428282230319L;
	private Model model;
	
	private JButton scanButton;
	private JToolBar toolBar;
	
	private DefaultTableModel deviceTableModel;
	private JTable deviceTable;
	private Vector<Object> columnNames;

	
	public RemoteDevicesPanel()
	{
		super(true);	//enable double-buffering
		
		//init vars
		model = null;
		columnNames = new Vector<Object>();
        columnNames.add(new String("Device Name"));
        columnNames.add(new String("Device Address"));
        columnNames.add(new String("Device Class"));
        columnNames.add(new String("Services"));

		//setup GUI
		GridBagLayout layout = new GridBagLayout();
        GridBagConstraints constraints = new GridBagConstraints();
        setLayout(layout);

        toolBar = new JToolBar();
        toolBar.setFloatable(false);
        scanButton = new JButton("Scan");
        toolBar.add(scanButton);
        constraints.gridx = 0;
        constraints.gridy = 0;
        constraints.gridwidth = 1;
        constraints.gridheight = 1;
        constraints.fill = GridBagConstraints.HORIZONTAL;
        constraints.ipadx = 0;
        constraints.ipady = 0;
        constraints.insets = new Insets(0, 0, 0, 0);		//order is = top, left, bottom, right
        constraints.weightx = 1;	//at least one component's weight values have to be set per row; otherwise, all the components clump together in the middle
        constraints.weighty = 0;
        constraints.anchor = GridBagConstraints.NORTHWEST;
        layout.setConstraints(toolBar, constraints);
        add(toolBar);
        
		//create the table
        Vector<Vector<Object>> rowVector = null;
        deviceTableModel = new DefaultTableModel(rowVector, columnNames);
		deviceTable = new JTable(deviceTableModel);
		deviceTable.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		JScrollPane deviceTableScrollPane = new JScrollPane(deviceTable);
		deviceTable.setFillsViewportHeight(true);
		
		constraints.gridx = 0;
		constraints.gridy = 1;
		constraints.gridwidth = 1;
		constraints.gridheight = 1;
		constraints.fill = GridBagConstraints.BOTH;
		constraints.ipadx = 0;
		constraints.ipady = 0;
		constraints.insets = new Insets(0, 5, 0, 5);		//order is = top, left, bottom, right
		constraints.weightx = 1;	//at least one component's weight values have to be set per row; otherwise, all the components clump together in the middle
		constraints.weighty = 1;
		constraints.anchor = GridBagConstraints.NORTHWEST;
		layout.setConstraints(deviceTableScrollPane, constraints);
		add(deviceTableScrollPane);
	}
	

	public void init(Model model, Controller controller)
	{
		this.model = model;
		
		//add event listeners
		scanButton.addActionListener(controller.getRemoteDevicesController());
		deviceTable.addMouseListener(controller.getRemoteDevicesController());
	}

	
	public void update()
	{
		//refresh the info to be displayed
		
		//check for presence of local BT device
		if (model.getDeviceStatus().equals(DeviceStatus.DEVICE_NOT_PRESENT))
		{
			//disable everything
			scanButton.setEnabled(false);
			deviceTable.setEnabled(false);
			return;
		}
		
		
		//add table data
		Vector<Vector<Object>> rowVector = model.getRemoteDevicesTableData();
		deviceTable.clearSelection();
		deviceTableModel.setDataVector(rowVector, columnNames);
		
		//highlight the table row with the currently selected device (if any)
		int selectedRow = model.getSelectedDeviceTableRow();
		if (selectedRow >= 0)
			deviceTable.setRowSelectionInterval(selectedRow, selectedRow);
	}

	
	//getters
	public JTable getDeviceTable() {
		return deviceTable;
	}

	public JButton getScanButton() {
		return scanButton;
	}
}
