/*
    This file is part of BlueJoy.

    BlueJoy is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    BlueJoy is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with BlueJoy.  If not, see <http://www.gnu.org/licenses/>.
*/

package org.bluejoy.view;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.io.File;

import javax.swing.ImageIcon;
import javax.swing.JLabel;
import javax.swing.JPanel;

import org.bluejoy.controller.Controller;
import org.bluejoy.model.Model;
import org.bluejoy.model.Model.DeviceStatus;



public class StatusBarPanel extends JPanel
{
	private static final long serialVersionUID = -8365255256126388470L;
	private Model model;
	
	private JLabel deviceSelectedLabel;
	private JLabel btStatusImageLabel;
	private ImageIcon btStatusGreenIcon;
	private ImageIcon btStatusYellowIcon;
	private ImageIcon btStatusRedIcon;
	private JLabel btStatusTextLabel;
	private ImageIcon cornerIcon;
	private JLabel cornerLabel;


	public StatusBarPanel()
	{
		super(true);	//enable double-buffering
		
		model = null;
		
		//init vars
		btStatusGreenIcon = new ImageIcon("." + File.separator + "resources" + File.separator + "green_dot.png");
		btStatusYellowIcon = new ImageIcon("." + File.separator + "resources" + File.separator + "yellow_dot.png");
		btStatusRedIcon = new ImageIcon("." + File.separator + "resources" + File.separator + "red_dot.png");
		
		//set up GUI
		GridBagLayout layout = new GridBagLayout();
        GridBagConstraints constraints = new GridBagConstraints();
        setLayout(layout);
        
        deviceSelectedLabel = new JLabel("No device selected");
        constraints.gridx = 0;
        constraints.gridy = 0;
        constraints.gridwidth = 1;
        constraints.gridheight = 1;
        constraints.fill = GridBagConstraints.NONE;
        constraints.ipadx = 0;
        constraints.ipady = 0;
        constraints.insets = new Insets(0, 5, 5, 0);		//order is = top, left, bottom, right
        constraints.weightx = 1;	//at least one component's weight values have to be set per row; otherwise, all the components clump together in the middle
        constraints.weighty = 1;
        constraints.anchor = GridBagConstraints.SOUTHWEST;
        layout.setConstraints(deviceSelectedLabel, constraints);
        add(deviceSelectedLabel);
		
        btStatusImageLabel = new JLabel(btStatusGreenIcon);
        constraints.gridx = 1;
        constraints.gridy = 0;
        constraints.gridwidth = 1;
        constraints.gridheight = 1;
        constraints.fill = GridBagConstraints.NONE;
        constraints.ipadx = 0;
        constraints.ipady = 0;
        constraints.insets = new Insets(5, 5, 5, 5);		//order is = top, left, bottom, right
        constraints.weightx = 0; //don't grow horizontally
        constraints.weighty = 0; //don't grow vertically
        constraints.anchor = GridBagConstraints.SOUTHEAST;
        layout.setConstraints(btStatusImageLabel, constraints);
        add(btStatusImageLabel);
		
        btStatusTextLabel = new JLabel("BT device is idle");
        btStatusTextLabel.setPreferredSize(new Dimension(125, 16));
        constraints.gridx = 2;
        constraints.gridy = 0;
        constraints.gridwidth = 1;
        constraints.gridheight = 1;
        constraints.fill = GridBagConstraints.NONE;
        constraints.ipadx = 0;
        constraints.ipady = 0;
        constraints.insets = new Insets(0, 5, 5, 5);		//order is = top, left, bottom, right
        constraints.weightx = 0; //don't grow horizontally
        constraints.weighty = 0; //don't grow vertically
        constraints.anchor = GridBagConstraints.SOUTHEAST;
        layout.setConstraints(btStatusTextLabel, constraints);
        add(btStatusTextLabel);
        
        cornerIcon = new ImageIcon(".\\resources\\corner_icon.png");
        cornerLabel = new JLabel(cornerIcon);
        constraints.gridx = 3;
        constraints.gridy = 0;
        constraints.gridwidth = 1;
        constraints.gridheight = 1;
        constraints.fill = GridBagConstraints.NONE;
        constraints.ipadx = 0;
        constraints.ipady = 0;
        constraints.insets = new Insets(0, 5, 0, 0);		//order is = top, left, bottom, right
        constraints.weightx = 0; 	//don't grow horizontally
        constraints.weighty = 0;	//don't grow vertically
        constraints.anchor = GridBagConstraints.SOUTHEAST;
        layout.setConstraints(cornerLabel, constraints);
        add(cornerLabel);
        
		//setBackground(SystemColor.control);
		//setPreferredSize(new Dimension(630, 35));
	}
	
	
	public void init(Model model, Controller controller)
	{
		this.model = model;
	}
	
	
	protected void paintComponent(Graphics g)
	{
		super.paintComponent(g);
		
		int y = 0;
		g.setColor(new Color(156, 154, 140));
		g.drawLine(0, y, getWidth(), y);
		y++;
		g.setColor(new Color(196, 194, 183));
		g.drawLine(0, y, getWidth(), y);
		y++;
		g.setColor(new Color(218, 215, 201));
		g.drawLine(0, y, getWidth(), y);
		y++;
		g.setColor(new Color(233, 231, 217));
		g.drawLine(0, y, getWidth(), y);
		
		y = getHeight() - 3;
		g.setColor(new Color(233, 232, 218));
		g.drawLine(0, y, getWidth(), y);
		y++;
		g.setColor(new Color(233, 231, 216));
		g.drawLine(0, y, getWidth(), y);
		y = getHeight() - 1;
		g.setColor(new Color(221, 221, 220));
		g.drawLine(0, y, getWidth(), y);
	}
	
	
	public void update()
	{
		//refresh the info to be displayed
		//device selected
		if (model.getSelectedDevice() != null)
			deviceSelectedLabel.setText("Device selected = " + model.getSelectedDevice().getDeviceName());
		else
			deviceSelectedLabel.setText("No device selected");
		
		//status icon and text
		if (model.getDeviceStatus().equals(DeviceStatus.IDLE))						//idle
		{
			btStatusImageLabel.setIcon(btStatusGreenIcon);
			btStatusTextLabel.setText("BT device is idle");
		}
		else if (model.getDeviceStatus().equals(DeviceStatus.DEVICE_SEARCH))		//scanning
		{
			btStatusImageLabel.setIcon(btStatusRedIcon);
			btStatusTextLabel.setText("BT device is scanning");
		}
		else if (model.getDeviceStatus().equals(DeviceStatus.SERVICE_SEARCH))
		{
			btStatusImageLabel.setIcon(btStatusRedIcon);
			btStatusTextLabel.setText("BT device is scanning");
		}
		else if (model.getDeviceStatus().equals(DeviceStatus.FTP_TRANSFER))			//transferring
		{
			btStatusImageLabel.setIcon(btStatusRedIcon);
			btStatusTextLabel.setText("BT device is transferring files");
		}
		else if (model.getDeviceStatus().equals(DeviceStatus.OPP_TRANSFER))
		{
			btStatusImageLabel.setIcon(btStatusRedIcon);
			btStatusTextLabel.setText("BT device is transferring files");
		}
		else if (model.getDeviceStatus().equals(DeviceStatus.SPP_TRANSFER))
		{
			btStatusImageLabel.setIcon(btStatusRedIcon);
			btStatusTextLabel.setText("BT device is transferring files");
		}
		else if (model.getDeviceStatus().equals(DeviceStatus.DEVICE_NOT_PRESENT))	//not available
		{
			btStatusImageLabel.setIcon(btStatusRedIcon);
			btStatusTextLabel.setText("BT device not present");
		}
		else if (model.getDeviceStatus().equals(DeviceStatus.UNINITIALIZED))
		{
			btStatusImageLabel.setIcon(btStatusRedIcon);
			btStatusTextLabel.setText("BT device is uninitialized");
		}
		
		
		//re-draw component
		update(getGraphics());
	}
}
