/*
    This file is part of BlueJoy.

    BlueJoy is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    BlueJoy is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with BlueJoy.  If not, see <http://www.gnu.org/licenses/>.
*/

package org.bluejoy.view;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.util.Observable;
import java.util.Observer;
import javax.swing.JFrame;
import javax.swing.JTabbedPane;

import org.bluejoy.controller.Controller;
import org.bluejoy.model.Model;
import org.bluejoy.model.Model.DeviceStatus;



public class View extends JFrame implements Observer
{
	private static final long serialVersionUID = 3665976791397791541L;
	private Model model;
	
	private JTabbedPane tabbedPane;
	private RemoteDevicesPanel remoteDevicesPanel;
	private FileTransferTxPanel fileTransferTxPanel;
	private FileTransferRxPanel fileTransferRxPanel;
	private StatusBarPanel statusBarPanel;
	
	
	
	//constructor
	public View(String name)
	{
		super(name);

		//init vars
		tabbedPane = new JTabbedPane();
		remoteDevicesPanel = new RemoteDevicesPanel();
		fileTransferTxPanel = new FileTransferTxPanel();
		fileTransferRxPanel = new FileTransferRxPanel();
		statusBarPanel = new StatusBarPanel();


		//setup GUI
		//TODO - figure out why setting the look and feel is broken
/*
		try
		{
			UIManager.setLookAndFeel("com.sun.java.swing.plaf.nimbus.NimbusLookAndFeel");
			UIManager.setLookAndFeel("com.sun.java.swing.plaf.windows.WindowsLookAndFeel");
			UIManager.setLookAndFeel("com.sun.java.swing.plaf.motif.MotifLookAndFeel");

		} 
		catch (Exception e) 
		{
			e.printStackTrace();
		}
*/

		
		add(tabbedPane, BorderLayout.CENTER);
		tabbedPane.addTab("Remote Devices", remoteDevicesPanel);
		tabbedPane.addTab("Send Files", fileTransferTxPanel);
		tabbedPane.addTab("Receive Files", fileTransferRxPanel);
		tabbedPane.setEnabled(false);
		add(statusBarPanel, BorderLayout.SOUTH);
		
		pack();	//set preferred sizes
		setDefaultCloseOperation(EXIT_ON_CLOSE);
		setSize(640, 480);
		setMinimumSize(new Dimension(640, 480));
		this.getRootPane().setDoubleBuffered(true);		//enable double-buffering
		setVisible(true);
	}
	
	public void init(Model model, Controller controller)
	{
		this.model = model;
		
		//init all panels
		remoteDevicesPanel.init(model, controller);
		fileTransferTxPanel.init(model, controller);
		fileTransferRxPanel.init(model, controller);
		statusBarPanel.init(model, controller);
		
		//observe model
		model.addObserver(this);
	}

	
	//Observer method
	public void update(Observable obs, Object arg)
	{
		if (obs instanceof Model)
		{
			//update the GUI
			update();
		}
	}
	
	
	//update GUI rendering
	public void update()
	{
		//CAUTION: the order in which the updates are done is important!
		//update embedded panels (i.e. panels embedded in the JTabbedPane)
		remoteDevicesPanel.update();
		fileTransferRxPanel.update();
		fileTransferTxPanel.update();
		
		
		//enable/disable other panes
		if ( (model.getDeviceStatus().equals(DeviceStatus.DEVICE_NOT_PRESENT) == false) && (model.getSelectedDevice() != null) )
			tabbedPane.setEnabled(true);
		else
			tabbedPane.setEnabled(false);
		
		//update this panel
		paintComponents(getGraphics());		//similar effect to "update(getGraphics());"
		statusBarPanel.update();
	}
	

	//getters
	public JTabbedPane getTabbedPane() {
		return tabbedPane;
	}
	
	public RemoteDevicesPanel getRemoteDevicesPanel() {
		return remoteDevicesPanel;
	}

	public StatusBarPanel getStatusBarPanel() {
		return statusBarPanel;
	}
	
	public FileTransferTxPanel getFileTransferTxPanel() {
		return fileTransferTxPanel;
	}

	public FileTransferRxPanel getFileTransferRxPanel() {
		return fileTransferRxPanel;
	}
	

	//for testing the GUI layout
	public static void main(String[] args)
	{
		new View("Test View");
	}
}
